"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_api_decorators_1 = require("ts-api-decorators");
const HttpBinding_1 = require("../generators/Bindings/HttpBinding");
const AzFuncBindings_1 = require("../metadata/AzFuncBindings");
const TimerBinding_1 = require("../generators/Bindings/TimerBinding");
const BlobStorageBinding_1 = require("../generators/Bindings/BlobStorageBinding");
function registerApiProcessorsOnHandler(handlerMethod, processors) {
    ts_api_decorators_1.ManagedApiInternal.AddApiProcessorsToObject(processors, handlerMethod);
}
exports.registerApiProcessorsOnHandler = registerApiProcessorsOnHandler;
class ManagedApi extends ts_api_decorators_1.ManagedApi {
    constructor() {
        super(true);
        this.initialized = false;
    }
    static GetSingleton() {
        const globalSymbols = Object.getOwnPropertySymbols(global);
        const keyDefined = (globalSymbols.indexOf(ManagedApi.SINGLETON_KEY) > -1);
        if (!keyDefined) {
            global[ManagedApi.SINGLETON_KEY] = {
                managedApi: new ManagedApi(),
            };
        }
        return global[ManagedApi.SINGLETON_KEY].managedApi;
    }
    static AzureTrigger(descr) {
        const singleton = ManagedApi.GetSingleton();
        return (context) => __awaiter(this, void 0, void 0, function* () {
            try {
                if (!singleton.initialized) {
                    yield singleton.init();
                }
                const { method, invocationParams } = singleton.resolveInvocationParamsForContext(descr.triggerType, context);
                if (!descr.methods.find(m => m === method)) {
                    throw new Error(`Invalid method for handler: ${method}`);
                }
                const handler = singleton.getHandler(method, descr.route);
                const result = yield handler.wrappedHandler(invocationParams);
                if (!context.res && result.statusCode >= 300) {
                    // TODO: Need to handle error responses in non-http functions
                    if (typeof result.body === 'string') {
                        throw new ts_api_decorators_1.HttpError(result.body, result.statusCode);
                    }
                    throw new ts_api_decorators_1.HttpError('Unknown Error', result.statusCode);
                }
                for (const bindingDef of context.bindingDefinitions) {
                    if (bindingDef.direction !== 'out') {
                        continue;
                    }
                    let assignValue = result.body;
                    if (bindingDef.name === AzFuncBindings_1.AzFuncBindingNameReturn) {
                        assignValue = result.body;
                    }
                    else if (typeof result.body[bindingDef.name] === 'undefined') {
                        throw new Error(`Could not find output binding ${bindingDef.name} of type ${bindingDef.type} on result`);
                    }
                    else {
                        assignValue = result.body[bindingDef.name];
                    }
                    if (bindingDef.type === 'http') {
                        context.res = {
                            status: result.statusCode,
                            body: assignValue,
                            headers: this.getHeadersObject(result.headers),
                        };
                    }
                    else {
                        context.bindings[bindingDef.name] = assignValue;
                    }
                }
            }
            catch (e) {
                const response = {
                    status: 500,
                    body: { message: 'Internal server error' }
                };
                if (e.statusCode) {
                    response.status = e.statusCode;
                }
                if (e.message) {
                    response.body.message = e.message;
                }
                if (response.status === 500) {
                    debugger;
                }
                context.res = response;
            }
        });
    }
    static getHeadersObject(headers) {
        if (!headers) {
            return {};
        }
        const responseHeaders = {};
        for (const header in Object.keys(headers)) {
            const origHeader = headers[header];
            if (typeof origHeader === 'string') {
                responseHeaders[header] = origHeader;
            }
            else if (Array.isArray(origHeader)) {
                responseHeaders[header] = origHeader.join(', ');
            }
            else {
                throw new Error('Could not serialize response headers');
            }
        }
        return responseHeaders;
    }
    getHandler(method, route) {
        if (!this.handlers.has(method)) {
            throw new Error(`Could not find handler with API method: ${method}`);
        }
        const methodGroup = this.handlers.get(method);
        if (!methodGroup.has(route)) {
            throw new Error(`Could not find handler for route: ${route}`);
        }
        return methodGroup.get(route);
    }
    resolveInvocationParamsForContext(triggerType, context) {
        switch (triggerType) {
            case "httpTrigger" /* HttpTrigger */:
                return HttpBinding_1.HttpBindingTriggerFactory.GetInvocationParams(context);
            case "timerTrigger" /* TimerTrigger */:
                return TimerBinding_1.TimerBindingTriggerFactory.GetInvocationParams(context);
            case "blobTrigger" /* BlobTrigger */:
                return BlobStorageBinding_1.BlobStorageBindingTriggerFactory.GetInvocationParams(context);
            default:
                throw new Error(`Unknown Azure trigger type: ${triggerType}`);
        }
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.handlers = this.initHandlers();
            this.initialized = true;
        });
    }
    getHandlerWrapper(instance) {
        throw new Error('Method Not Implemented');
    }
    setHeader(name, value) {
        throw new Error('Method Not Implemented');
    }
}
exports.ManagedApi = ManagedApi;
ManagedApi.SINGLETON_KEY = Symbol.for("MB.ts-api-decorators-azure-function.ManagedApi");
//# sourceMappingURL=ManagedApi.js.map