"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CliCommand_1 = require("ts-api-decorators/dist/command/CliCommand");
const OutputFileGenerator_1 = require("ts-api-decorators/dist/generators/OutputFileGenerator");
const FunctionFileGenerator_1 = require("../generators/FunctionFileGenerator");
const FunctionJsonFileGenerator_1 = require("../generators/FunctionJsonFileGenerator");
const HttpBinding_1 = require("../generators/Bindings/HttpBinding");
const FunctionHostFileGenerator_1 = require("../generators/FunctionHostFileGenerator");
const FunctionExtensionJsonFileGenerator_1 = require("../generators/FunctionExtensionJsonFileGenerator");
const TimerBinding_1 = require("../generators/Bindings/TimerBinding");
const transformer_1 = require("../transformer");
const BlobStorageBinding_1 = require("../generators/Bindings/BlobStorageBinding");
const ApiParser_1 = require("ts-api-decorators/dist/Util/ApiParser");
const RouteReducer_1 = require("../Util/RouteReducer");
class AzureFunctionGenerateCommand extends CliCommand_1.CliCommand {
    constructor(program, additionalOpts) {
        super();
        this.program = program;
        this.additionalOpts = additionalOpts;
        if (!this.additionalOpts) {
            this.additionalOpts = {};
        }
        this.apiParser = new ApiParser_1.ApiParser();
        program
            .command('azfunc-generate <rootDir> <outDir>')
            .description('Output an Azure Function app', {
            'rootDir': 'The root directory or file of the program. If directory, should be the directory that contains tsConfig',
            'outDir': 'A directory to output the Azure function to',
        })
            .option('--tsconfig <file>', 'The tsconfig.json file to use when compiling')
            .option('--silent', 'Don\'t output information', false)
            .option('--verbose', 'Write extra output information', false)
            .option('--apiInfo <file>', 'File containing information for the API', 'package.json')
            .option('--httpPrefix <prefix>', 'The prefix path for all http bindings. Defaults to /api')
            .action((rootDir, outDir, options) => this.runCommand(Object.assign(Object.assign({}, options), { rootDir,
            outDir })));
    }
    runCommand(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const api = yield this.apiParser.parseApi(options, transformer_1.getTransformerArguments());
            if (!options.silent) {
                let summary = this.printExtractionSummary(options, api, this.apiParser.getTree());
                this.printSummary(summary);
            }
            const generatorOpts = {
                triggers: [
                    HttpBinding_1.HttpBindingTriggerFactory.GetBindingForMethod("GET" /* GET */),
                    HttpBinding_1.HttpBindingTriggerFactory.GetBindingForMethod("PUT" /* PUT */),
                    HttpBinding_1.HttpBindingTriggerFactory.GetBindingForMethod("POST" /* POST */),
                    HttpBinding_1.HttpBindingTriggerFactory.GetBindingForMethod("DELETE" /* DELETE */),
                    TimerBinding_1.TimerBindingTriggerFactory.GetBinding(),
                    BlobStorageBinding_1.BlobStorageBindingTriggerFactory.GetTriggerBinding(),
                ],
                params: [
                    TimerBinding_1.TimerBindingTriggerFactory.GetParamBinding(),
                    ...BlobStorageBinding_1.BlobStorageBindingParamFactory.GetParamBindings(),
                ],
                outputs: [
                    ...BlobStorageBinding_1.BlobStorageOutputBindingFactory.GetOutputBindings(),
                ],
            };
            const outGenerator = new OutputFileGenerator_1.OutputFileGenerator(options.outDir);
            const hostGenOpts = Object.assign(Object.assign({}, generatorOpts), { tsConfig: api.tsConfig });
            if (options.httpPrefix) {
                hostGenOpts.extensionOpts = {
                    http: {
                        routePrefix: options.httpPrefix === AzureFunctionGenerateCommand.HttpPrefixNoneStr
                            ? ''
                            : options.httpPrefix,
                    },
                };
            }
            const hostGen = new FunctionHostFileGenerator_1.FunctionHostFileGenerator(hostGenOpts);
            const extensionGen = new FunctionExtensionJsonFileGenerator_1.FunctionExtensionJsonFileGenerator();
            const functionGen = new FunctionFileGenerator_1.FunctionFileGenerator(Object.assign(Object.assign({}, generatorOpts), { tsConfig: api.tsConfig, tsConfigPath: api.tsConfigPath, libIncludePath: (_a = this.additionalOpts) === null || _a === void 0 ? void 0 : _a.alternateLibIncludePath }));
            const functionJsonGen = new FunctionJsonFileGenerator_1.FunctionJsonFileGenerator(generatorOpts);
            const routeNames = new Set();
            outGenerator.addOutputFile(hostGen.getFilename(), hostGen.forTree([api.tree]));
            outGenerator.addOutputFile(extensionGen.getFilename(), extensionGen.forTree([api.tree]));
            for (let [[method, route, ...bindings], routes] of RouteReducer_1.RouteReducer.reduceFunctionRoutesByPath(api)) {
                route = route.startsWith('/') ? route.substr(1) : route;
                const baseRouteName = `${route.replace(/[^-a-zA-Z0-9_]/g, '_').toString()}`;
                let routeName = baseRouteName;
                let attempt = 0;
                while (routeNames.has(routeName)) {
                    routeName = baseRouteName + '_' + ++attempt;
                }
                routeNames.add(routeName);
                outGenerator.addOutputFile(functionJsonGen.getFilenameForFunction(routeName), functionJsonGen.forTree(routes));
                outGenerator.addOutputFile(functionGen.getFilenameForFunction(routeName), functionGen.forTree(routes));
            }
            const written = yield outGenerator.generate();
            if (options.verbose) {
                console.log('Wrote output files: ');
                for (const f of written) {
                    console.log('\t', f);
                }
            }
        });
    }
    printSummary(summary) {
        console.log(summary instanceof Buffer ? summary.toString('utf8') : summary);
    }
    validateProgramOutputType(value, defaultValue) {
        switch (value) {
            case 'summary':
            case 'json':
            case 'swagger2':
                return value;
            case undefined:
                return defaultValue;
            default:
                throw new Error('Invalid value for `type`: ' + value);
        }
    }
}
exports.AzureFunctionGenerateCommand = AzureFunctionGenerateCommand;
AzureFunctionGenerateCommand.HttpPrefixNoneStr = '<none>';
//# sourceMappingURL=AzureFunctionGenerate.js.map