import { ApiMethodDecoratorReturnType } from "ts-api-decorators";
import { ApiMethodReturnType } from "ts-api-decorators/dist/apiManagement/ApiDefinition";
import { HandlerMethodModifierDecorator } from "ts-api-decorators/dist/transformer/HandlerMethodModifierDecorator";
import { HandlerMethodParameterDecorator } from "ts-api-decorators/dist/transformer/HandlerMethodParameterDecorator";
import { HandlerMethodDecorator } from "ts-api-decorators/dist/transformer/HandlerMethodDecorator";
declare abstract class BlobStorageMethodDecorators {
    /**
     * Azure Functions Blob Trigger
     * @param path The container to monitor. May be a blob name pattern.
     * @param connection The name of an app setting that contains the Storage connection string to use for this binding. If the app setting name begins with "AzureWebJobs", you can specify only the remainder of the name here. For example, if you set connection to "MyStorage", the Functions runtime looks for an app setting that is named "MyStorage." If you leave connection empty, the Functions runtime uses the default Storage connection string in the app setting that is named AzureWebJobsStorage.
     */
    static AzFuncBlob<T extends ApiMethodReturnType>(path: string, connection?: string): ApiMethodDecoratorReturnType<T>;
}
export declare abstract class BlobStorageParams {
    static readonly TransportTypeBlobInputParam = "inputBlob";
    static readonly TransportTypeBlobInputPropsParam = "inputBlobProps";
    static readonly TransportTypeBlobOutputParam = "outputblob";
    static AzFuncBlobParam(): ParameterDecorator;
    static AzFuncBlobParam(path: string, connection?: string): ParameterDecorator;
    static AzFuncBlobPropertiesParam(): ParameterDecorator;
    static AzFuncBlobOutput<T extends object>(outField: keyof T, path: string, connection?: string): ApiMethodDecoratorReturnType<T>;
}
export declare const AzFuncBlobMethodDecorator: (param: TimerHandler) => HandlerMethodDecorator;
export declare const AzFuncBlobParamDecorator: (param: TimerHandler) => HandlerMethodParameterDecorator;
export declare const AzFuncBlobModifierDecorator: (param: TimerHandler) => HandlerMethodModifierDecorator;
export declare const AzFuncBlob: typeof BlobStorageMethodDecorators.AzFuncBlob;
export declare const AzFuncBlobParam: typeof BlobStorageParams.AzFuncBlobParam;
export declare const AzFuncBlobPropertiesParam: typeof BlobStorageParams.AzFuncBlobPropertiesParam;
export declare const AzFuncBlobOutput: typeof BlobStorageParams.AzFuncBlobOutput;
export {};
