"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TransformerMetadata_1 = require("ts-api-decorators/dist/transformer/TransformerMetadata");
const AzFuncMetadata_1 = require("../../metadata/AzFuncMetadata");
const BlobStorageTrigger_1 = require("../../decorators/ExtensionDecorators/BlobStorage/BlobStorageTrigger");
class BlobStorageBindingTriggerFactory {
    static GetTriggerBinding() {
        return {
            triggerMethod: "blobTrigger" /* BlobTrigger */,
            triggerType: "blobTrigger" /* BlobTrigger */,
            getBindingForRoutes: routes => ([
                Object.assign({ type: "blobTrigger" /* BlobTrigger */, direction: 'in', name: 'blobTrigger' }, BlobStorageBindingTriggerFactory.getTriggerParamsForRoutes(routes)),
            ])
        };
    }
    static getTriggerParamsForRoutes(routes) {
        let path;
        let connection;
        for (const route of routes) {
            const p = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, TransformerMetadata_1.BuiltinMetadata.Route);
            if (p) {
                if (path) {
                    throw new Error('Can only declare one path per blob');
                }
                path = p;
            }
            const c = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.BlobStorageConnection);
            if (typeof c !== 'undefined') {
                if (typeof connection !== 'undefined') {
                    throw new Error('Can only declare one connection per blob');
                }
                connection = c;
            }
        }
        const result = {
            path,
        };
        if (typeof connection !== 'undefined') {
            result.connection = connection;
        }
        return result;
    }
    static GetInvocationParams(context) {
        if (!context.bindings.blobTrigger) {
            throw new Error('Context does not have http req');
        }
        return {
            method: "blobTrigger" /* BlobTrigger */,
            invocationParams: {
                queryParams: {},
                // TODO: Path params from the blob path once we figure out what's inside context.bindings.blobTrigger
                pathParams: this.GetPathParamsFromBlobBindingData(context.bindingData),
                headers: {},
                transportParams: {
                    context,
                    [BlobStorageTrigger_1.BlobStorageParams.TransportTypeBlobInputParam]: context.bindings.blobTrigger,
                    [BlobStorageTrigger_1.BlobStorageParams.TransportTypeBlobInputPropsParam]: context.bindingData.properties,
                },
            }
        };
    }
    static GetPathParamsFromBlobBindingData(bindingData) {
        const skipNames = [
            '$request',
            'metadata',
            'properties',
            'sys'
        ];
        const params = {};
        for (const param of Object.keys(bindingData)) {
            if (skipNames.indexOf(param) !== -1 || typeof bindingData[param] !== 'string') {
                continue;
            }
            params[param] = bindingData[param];
        }
        return params;
    }
}
exports.BlobStorageBindingTriggerFactory = BlobStorageBindingTriggerFactory;
class BlobStorageBindingParamFactory {
    static GetParamBindings() {
        return [
            {
                paramTypeId: BlobStorageTrigger_1.BlobStorageParams.TransportTypeBlobInputParam,
                getBindingForParam: (param, route) => {
                    if (!TransformerMetadata_1.getMetadataValueByDescriptor(route.children[param.parameterIndex].metadata, TransformerMetadata_1.BuiltinMetadata.Route)) {
                        return [];
                    }
                    return [
                        Object.assign({ type: "blob" /* Blob */, direction: 'in', name: `${BlobStorageTrigger_1.BlobStorageParams.TransportTypeBlobInputParam}-${typeof param.propertyKey === 'string' ? param.propertyKey : param.parameterIndex}` }, BlobStorageBindingParamFactory.getTriggerParamsForRoutes(route)),
                    ];
                },
            },
            {
                paramTypeId: BlobStorageTrigger_1.BlobStorageParams.TransportTypeBlobInputPropsParam,
                getBindingForParam: (param, route) => {
                    if (!TransformerMetadata_1.getMetadataValueByDescriptor(route.children[param.parameterIndex].metadata, TransformerMetadata_1.BuiltinMetadata.Route)) {
                        return [];
                    }
                    throw new Error('Blob Props with Path Not Implemented');
                },
            },
        ];
    }
    static getTriggerParamsForRoutes(route) {
        let path;
        let connection;
        const p = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, TransformerMetadata_1.BuiltinMetadata.Route);
        if (p) {
            path = p;
        }
        const c = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.BlobStorageConnection);
        if (typeof c !== 'undefined') {
            connection = c;
        }
        const result = {
            path,
        };
        if (typeof connection !== 'undefined') {
            result.connection = connection;
        }
        return result;
    }
}
exports.BlobStorageBindingParamFactory = BlobStorageBindingParamFactory;
class BlobStorageOutputBindingFactory {
    static GetOutputBindings() {
        return [
            {
                outputTypeId: BlobStorageTrigger_1.BlobStorageParams.TransportTypeBlobOutputParam,
                getBindingForOutput: (output, route) => {
                    const name = TransformerMetadata_1.getMetadataValueByDescriptor(output.metadata, AzFuncMetadata_1.AzFuncMetadata.OutField);
                    if (!name) {
                        throw new Error('Output binding must have a name');
                    }
                    const path = TransformerMetadata_1.getMetadataValueByDescriptor(output.metadata, TransformerMetadata_1.BuiltinMetadata.Route);
                    const connection = TransformerMetadata_1.getMetadataValueByDescriptor(output.metadata, AzFuncMetadata_1.AzFuncMetadata.BlobStorageConnection);
                    return [
                        {
                            type: "blob" /* Blob */,
                            direction: 'out',
                            name,
                            path,
                            connection,
                        },
                    ];
                },
            },
        ];
    }
}
exports.BlobStorageOutputBindingFactory = BlobStorageOutputBindingFactory;
//# sourceMappingURL=BlobStorageBinding.js.map