"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_api_decorators_1 = require("ts-api-decorators");
const __1 = require("../..");
const TrimLeft_1 = require("../../Util/TrimLeft");
const AzFuncBindings_1 = require("../../metadata/AzFuncBindings");
class HttpBindingTriggerFactory {
    static GetBindingForMethod(method) {
        return {
            triggerMethod: method,
            triggerType: "httpTrigger" /* HttpTrigger */,
            getBindingForRoutes: routes => ([
                // Input trigger binding
                {
                    type: "httpTrigger" /* HttpTrigger */,
                    direction: 'in',
                    name: __1.AzureFunctionParams.TransportTypeRequestParam,
                    route: HttpBindingTriggerFactory.RewriteRouteForAzureFunction(routes[0].route),
                    methods: routes.map(r => r.apiMethod.toLowerCase()),
                },
                // Output binding
                {
                    type: 'http',
                    direction: 'out',
                    name: AzFuncBindings_1.AzFuncBindingNameReturn,
                }
            ])
        };
    }
    static GetMethodForRequestContext(context) {
        switch (context.req.method.toUpperCase()) {
            case "GET" /* GET */:
                return "GET" /* GET */;
            case "DELETE" /* DELETE */:
                return "DELETE" /* DELETE */;
            case "POST" /* POST */:
                return "POST" /* POST */;
            case "PUT" /* PUT */:
                return "PUT" /* PUT */;
            default:
                throw new Error(`Unknown http request method: ${context.req.method}`);
        }
    }
    static GetInvocationParams(context) {
        if (!context.req) {
            throw new Error('Context does not have http req');
        }
        const method = this.GetMethodForRequestContext(context);
        const contentType = this.getHeader(context.req.headers, ts_api_decorators_1.ApiStdHeaderName.ContentType);
        const contentLength = this.getHeader(context.req.headers, ts_api_decorators_1.ApiStdHeaderName.ContentLength);
        const parsedContentType = ts_api_decorators_1.parseApiMimeType(contentType);
        return {
            method,
            invocationParams: {
                queryParams: context.req.query,
                pathParams: context.req.params,
                headers: context.req.headers,
                bodyContents: ((typeof contentType !== 'undefined' && Number(contentLength) > 0)
                    ?
                        {
                            contentsStream: context.req.body,
                            // TODO: Add text encoding?
                            readStreamToStringAsync: () => context.req.rawBody,
                            readStreamToStringCb: (cb) => cb(null, context.req.rawBody),
                            streamContentsMimeRaw: contentType,
                            streamContentsMimeType: parsedContentType.mimeType,
                        }
                    : undefined),
                transportParams: {
                    context,
                },
            }
        };
    }
    static getHeader(headers, headerName) {
        headerName = headerName.toLowerCase();
        return headers[headerName];
    }
    static RewriteRouteForAzureFunction(route) {
        return TrimLeft_1.trimLeft(ts_api_decorators_1.ManagedApi.GetRouteTokens(route)
            .map(t => {
            if (typeof t === 'string') {
                return t;
            }
            let outStr = t.prefix ? t.prefix : '';
            if (t.modifier && t.modifier !== '?') {
                console.warn('Unhandled modifier: ' + t.modifier);
            }
            outStr += `{${t.name}:regex(${t.pattern})${t.modifier ? t.modifier : ''}}`;
            if (t.suffix) {
                outStr += t.suffix;
            }
            return outStr;
        })
            .join(''), '/');
    }
}
exports.HttpBindingTriggerFactory = HttpBindingTriggerFactory;
//# sourceMappingURL=HttpBinding.js.map