"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TransformerMetadata_1 = require("ts-api-decorators/dist/transformer/TransformerMetadata");
const AzFuncMetadata_1 = require("../../metadata/AzFuncMetadata");
class TimerBindingTriggerFactory {
    static GetBinding() {
        return {
            triggerMethod: "timerTrigger" /* TimerTrigger */,
            triggerType: "timerTrigger" /* TimerTrigger */,
            getBindingForRoutes: routes => ([
                Object.assign({ type: "timerTrigger" /* TimerTrigger */, direction: 'in', name: 'timer' }, TimerBindingTriggerFactory.getTriggerParamsForRoutes(routes)),
            ])
        };
    }
    static GetParamBinding() {
        return {
            paramTypeId: 'timer',
        };
    }
    static getTriggerParamsForRoutes(routes) {
        let schedule;
        let runOnStartup;
        let useMonitor;
        for (const route of routes) {
            const s = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.TimerSchedule);
            if (s) {
                if (schedule) {
                    throw new Error('Can only declare one schedule per timer');
                }
                schedule = s;
            }
            const r = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.TimerRunOnStartup);
            if (typeof r !== 'undefined') {
                if (typeof runOnStartup !== 'undefined') {
                    throw new Error('Can only declare value for runOnStartup per timer');
                }
                runOnStartup = r;
            }
            const u = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.TimerUseMonitor);
            if (typeof u !== 'undefined') {
                if (typeof useMonitor !== 'undefined') {
                    throw new Error('Can only declare value for useMonitor per timer');
                }
                useMonitor = u;
            }
        }
        const result = {
            schedule,
        };
        if (typeof runOnStartup !== 'undefined') {
            result.runOnStartup = runOnStartup;
        }
        if (typeof useMonitor !== 'undefined') {
            result.useMonitor = useMonitor;
        }
        return result;
    }
    static GetInvocationParams(context) {
        if (!context.bindings.timer) {
            throw new Error('Context does not have http req');
        }
        return {
            method: "timerTrigger" /* TimerTrigger */,
            invocationParams: {
                queryParams: {},
                pathParams: {},
                headers: {},
                transportParams: {
                    context,
                    timer: context.bindings.timer,
                },
            }
        };
    }
}
exports.TimerBindingTriggerFactory = TimerBindingTriggerFactory;
//# sourceMappingURL=TimerBinding.js.map