"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HandlerTree_1 = require("ts-api-decorators/dist/transformer/HandlerTree");
const AzFuncExtension_1 = require("../../metadata/AzFuncExtension");
const TransformerMetadata_1 = require("ts-api-decorators/dist/transformer/TransformerMetadata");
const AzFuncMetadata_1 = require("../../metadata/AzFuncMetadata");
class FunctionExtensionJsonFileGenerator {
    forTree(routes) {
        if (routes.length === 0) {
            throw new Error('Azure Function Generator requires at least one route');
        }
        return (currentFile) => __awaiter(this, void 0, void 0, function* () {
            const result = yield this.generateExtensionJson(currentFile, routes);
            if (result) {
                return Buffer.from(result);
            }
        });
    }
    generateExtensionJson(currentFile, routes) {
        const extFile = this.getExtensionsForRoutes(routes);
        if (extFile.length === 0) {
            return;
        }
        return JSON.stringify(extFile, undefined, FunctionExtensionJsonFileGenerator.JSON_PRETTY_PRINT_SPACE);
    }
    getExtensionsForRoutes(routes) {
        const extensionIds = new Set();
        for (const route of routes) {
            this.addExtensionIds(route, extensionIds);
            for (const node of HandlerTree_1.WalkTree(route)) {
                this.addExtensionIds(node, extensionIds);
            }
        }
        return Array.from(extensionIds).map(id => {
            const ext = AzFuncExtension_1.AzFuncExtension.GetExtensionForId(id);
            return {
                id: ext.id,
                version: ext.version,
            };
        });
    }
    addExtensionIds(route, extensionIds) {
        const meta = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.ExtensionBundle);
        if (meta && !extensionIds.has(meta)) {
            extensionIds.add(meta);
        }
    }
    getFilename() {
        return `${FunctionExtensionJsonFileGenerator.FILE_NAME}`;
    }
}
exports.FunctionExtensionJsonFileGenerator = FunctionExtensionJsonFileGenerator;
FunctionExtensionJsonFileGenerator.FILE_NAME = 'extensions.json';
FunctionExtensionJsonFileGenerator.JSON_PRETTY_PRINT_SPACE = 4;
//# sourceMappingURL=FunctionExtensionJsonFileGenerator.js.map