"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const GeneratorUtil_1 = require("ts-api-decorators/dist/Util/GeneratorUtil");
const HandlerTree_1 = require("ts-api-decorators/dist/transformer/HandlerTree");
const AzFuncExtension_1 = require("../../metadata/AzFuncExtension");
const TransformerMetadata_1 = require("ts-api-decorators/dist/transformer/TransformerMetadata");
const AzFuncMetadata_1 = require("../../metadata/AzFuncMetadata");
class FunctionHostFileGenerator {
    constructor(opts) {
        this.opts = opts;
        this.triggers = new Map();
        this.params = new Map();
        for (const t of opts.triggers) {
            this.triggers.set(t.triggerMethod, t);
        }
        for (const p of opts.params) {
            this.params.set(p.paramTypeId, p);
        }
    }
    forTree(routes) {
        if (routes.length === 0) {
            throw new Error('Azure Function Generator requires at least one route');
        }
        return (currentFile) => __awaiter(this, void 0, void 0, function* () { return Buffer.from(yield this.generateHostJson(currentFile, routes)); });
    }
    generateHostJson(currentFile, routes) {
        const hostfile = {
            version: FunctionHostFileGenerator.DEFAULT_FUNCTIONS_VERSION,
        };
        if (typeof this.opts.setWatchDirectories !== 'boolean' || this.opts.setWatchDirectories) {
            hostfile.watchDirectories = this.getWatchedDirectories(currentFile);
        }
        const extensions = this.getExtensionsForRoutes(routes);
        if (extensions) {
            hostfile.extensionBundle = AzFuncExtension_1.AzFuncExtension.ExtensionBundle;
        }
        if (this.opts.extensionOpts) {
            hostfile.extensions = this.opts.extensionOpts;
        }
        return JSON.stringify(hostfile, undefined, FunctionHostFileGenerator.JSON_PRETTY_PRINT_SPACE);
    }
    getExtensionsForRoutes(routes) {
        const extensionIds = new Set();
        for (const route of routes) {
            this.addExtensionIds(route, extensionIds);
            for (const node of HandlerTree_1.WalkTree(route)) {
                this.addExtensionIds(node, extensionIds);
            }
        }
        return Array.from(extensionIds).map(id => {
            const ext = AzFuncExtension_1.AzFuncExtension.GetExtensionForId(id);
            return {
                id: ext.id,
                version: ext.version,
            };
        });
    }
    addExtensionIds(route, extensionIds) {
        const meta = TransformerMetadata_1.getMetadataValueByDescriptor(route.metadata, AzFuncMetadata_1.AzFuncMetadata.ExtensionBundle);
        if (meta && !extensionIds.has(meta)) {
            extensionIds.add(meta);
        }
    }
    getWatchedDirectories(currentFile) {
        const dirs = [];
        const outputDir = this.getRelativePathOfOutputDirectory(currentFile);
        if (outputDir) {
            dirs.push(outputDir);
        }
        return dirs;
    }
    getRelativePathOfOutputDirectory(currentFile) {
        if (!this.opts.tsConfig) {
            return;
        }
        let outDir;
        if (this.opts.tsConfig.options.outDir) {
            outDir = this.opts.tsConfig.options.outDir;
        }
        if (this.opts.tsConfig.options.sourceRoot) {
            outDir = this.opts.tsConfig.options.sourceRoot;
        }
        if (outDir) {
            return GeneratorUtil_1.GeneratorUtil.NormalizePathSlashes(path.relative(path.dirname(currentFile), outDir));
        }
    }
    getFilename() {
        return `${FunctionHostFileGenerator.FILE_NAME}`;
    }
}
exports.FunctionHostFileGenerator = FunctionHostFileGenerator;
FunctionHostFileGenerator.FILE_NAME = 'host.json';
FunctionHostFileGenerator.DEFAULT_FUNCTIONS_VERSION = '2.0';
FunctionHostFileGenerator.JSON_PRETTY_PRINT_SPACE = 4;
//# sourceMappingURL=FunctionHostFileGenerator.js.map