"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class AzFuncExtension {
    static GetExtensionForId(idStr) {
        const extension = AzFuncExtension._Extensions.find(s => s.id === idStr);
        if (!extension) {
            throw new Error(`No extension with id: '${idStr}'`);
        }
        return extension;
    }
    static GetExtensionForBinding(binding) {
        const extension = AzFuncExtension._Extensions.find(s => s.bindings && !!s.bindings.find(b => b === binding));
        if (!extension) {
            if (AzFuncExtension._BuiltinBindings.find(b => b === binding)) {
                return undefined;
            }
            throw new Error(`No binding with id: '${binding}'`);
        }
        return extension;
    }
    static IsTriggerTypeBinding(binding) {
        return !!AzFuncExtension._TriggerBindings.find(b => b === binding);
    }
}
exports.AzFuncExtension = AzFuncExtension;
AzFuncExtension.ExtensionBundle = {
    "id": "Microsoft.Azure.Functions.ExtensionBundle",
    "version": "1.0",
};
AzFuncExtension.AzureStorage = {
    "id": "Microsoft.Azure.WebJobs.Extensions.Storage",
    "version": "3.0.10",
    "name": "AzureStorage",
    "bindings": [
        "blobTrigger" /* BlobTrigger */,
        "queue" /* Queue */,
        "queuetrigger" /* QueueTrigger */,
        "table" /* Table */,
    ]
};
AzFuncExtension.ServiceBus = {
    "id": "Microsoft.Azure.WebJobs.Extensions.ServiceBus",
    "version": "3.2.0",
    "name": "ServiceBus",
    "bindings": [
        "servicebustrigger" /* ServiceBusTrigger */,
        "servicebus" /* ServiceBus */,
    ]
};
AzFuncExtension.EventHubs = {
    "id": "Microsoft.Azure.WebJobs.Extensions.EventHubs",
    "version": "3.0.6",
    "name": "EventHubs",
    "bindings": [
        "eventhubtrigger" /* EventHubTrigger */,
        "eventhub" /* EventHub */,
    ]
};
AzFuncExtension.SendGrid = {
    "id": "Microsoft.Azure.WebJobs.Extensions.SendGrid",
    "version": "3.0.0",
    "name": "SendGrid",
    "bindings": [
        "sendgrid" /* SendGrid */,
    ]
};
AzFuncExtension.DurableTask = {
    "id": "Microsoft.Azure.WebJobs.Extensions.DurableTask",
    "version": "1.8.3",
    "name": "DurableTask",
    "bindings": [
        "activitytrigger" /* ActivityTrigger */,
        "orchestrationtrigger" /* OrchestionTrigger */,
        "orchestrationclient" /* OrchestrationClient */,
    ]
};
AzFuncExtension.EventGrid = {
    "id": "Microsoft.Azure.WebJobs.Extensions.EventGrid",
    "version": "2.1.0",
    "name": "EventGrid",
    "bindings": [
        "eventgridtrigger" /* EventGridTrigger */,
    ]
};
AzFuncExtension.CosmosDB = {
    "id": "Microsoft.Azure.WebJobs.Extensions.CosmosDB",
    "version": "3.0.5",
    "name": "CosmosDB",
    "bindings": [
        "cosmosdbtrigger" /* CosmosDBTrigger */,
        "cosmosdb" /* CosmosDB */,
    ]
};
AzFuncExtension.Twilio = {
    "id": "Microsoft.Azure.WebJobs.Extensions.Twilio",
    "version": "3.0.0",
    "name": "Twilio",
    "bindings": [
        "twiliosms" /* TwilioSms */,
    ]
};
AzFuncExtension.SignalRService = {
    "id": "Microsoft.Azure.WebJobs.Extensions.SignalRService",
    "version": "1.0.0",
    "name": "SignalR",
    "bindings": [
        "signalr" /* SignalR */,
        "signalrconnectioninfo" /* SignalRConnectionInfo */,
    ]
};
AzFuncExtension._Extensions = [
    AzFuncExtension.ExtensionBundle,
    AzFuncExtension.AzureStorage,
    AzFuncExtension.ServiceBus,
    AzFuncExtension.EventHubs,
    AzFuncExtension.SendGrid,
    AzFuncExtension.DurableTask,
    AzFuncExtension.EventGrid,
    AzFuncExtension.CosmosDB,
    AzFuncExtension.Twilio,
    AzFuncExtension.SignalRService,
];
AzFuncExtension._BuiltinBindings = [
    "httpTrigger" /* HttpTrigger */,
    "http" /* Http */,
    "timerTrigger" /* TimerTrigger */,
];
AzFuncExtension._TriggerBindings = [
    "httpTrigger" /* HttpTrigger */,
    "timerTrigger" /* TimerTrigger */,
    "blobTrigger" /* BlobTrigger */,
    "queuetrigger" /* QueueTrigger */,
    "servicebustrigger" /* ServiceBusTrigger */,
    "eventhubtrigger" /* EventHubTrigger */,
    "activitytrigger" /* ActivityTrigger */,
    "orchestrationtrigger" /* OrchestionTrigger */,
    "eventgridtrigger" /* EventGridTrigger */,
    "cosmosdbtrigger" /* CosmosDBTrigger */,
];
//# sourceMappingURL=AzFuncExtension.js.map